@@ -48,6 +48,24 @@ namespace rcsc {
 */
 class Body_KickTwoStep
     : public BodyAction {
+public:
+
+    struct SubTarget {
+        Vector2D ball_pos_;
+        Vector2D ball_vel_;
+        double opp_dist2_;
+
+        SubTarget( const Vector2D & ball_pos,
+                   const Vector2D & ball_vel,
+                   const double & opp_dist2 )
+            : ball_pos_( ball_pos )
+            , ball_vel_( ball_vel )
+            , opp_dist2_( opp_dist2 )
+          { }
+    };
+
+    static const double DEFAULT_MIN_DIST2;
+
 private:
     //! target point where ball should reach or pass through
     const Vector2D M_target_point;
@@ -82,15 +100,19 @@ public:
       \brief check if opponent is kickable at rel_pos
       \param agent agent itself
       \param rel_pos considered position relative to current agent pos
+      \param min_dist2 minimum distance to the opponent
       \return true if kickable opponent exists
     */
     static
     bool is_opp_kickable( const PlayerAgent * agent,
-                          const Vector2D & rel_pos );
+                          const Vector2D & rel_pos,
+                          double * min_dist2 );
 
     /*!
       \brief simulate one kick
       \param achieved_vel max reachable ball velocity.
+      \param kick_power varialble to store the estimated last kick power
+      \param opp_dist2 varialble to store the estimated squared distance to the nearest opponent
       \param target_rpos target position relative to current agent position
       \param first_speed desired ball first speed
       \param my_rpos estimated agent position relative to current agent position
@@ -104,6 +126,8 @@ public:
     */
     static
     bool simulate_one_kick( Vector2D * achieved_vel,
+                            double * kick_power,
+                            double * opp_dist2,
                             const Vector2D & target_rpos,
                             const double & first_speed,
                             const Vector2D & my_rpos,
