@@ -39,7 +39,7 @@
 
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
