@@ -29,8 +29,8 @@
 
 /////////////////////////////////////////////////////////////////////
 
-#ifndef RCSC_ACTION_INTENTION_DRIBBLE_H
-#define RCSC_ACTION_INTENTION_DRIBBLE_H
+#ifndef RCSC_ACTION_INTENTION_DRIBBLE_TEST_H
+#define RCSC_ACTION_INTENTION_DRIBBLE_TEST_H
 
 #include <rcsc/player/soccer_intention.h>
 #include <rcsc/geom/vector_2d.h>
