@@ -38,7 +38,7 @@
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/soccer_action.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
