@@ -47,9 +47,6 @@ namespace rcsc {
 class Neck_ScanField
     : public NeckAction {
 private:
-    static GameTime S_last_calc_time;
-    static ViewWidth S_last_calc_view_width;
-    static AngleDeg S_cached_target_angle;
 
 public:
     /*!
@@ -75,11 +72,8 @@ public:
       }
 
 private:
-    AngleDeg calcAngle( const PlayerAgent * agent,
-                        const AngleDeg & left_start,
-                        const double & scan_range,
-                        const Vector2D & base_point,
-                        const double & next_view_width );
+    AngleDeg calcAngle( const PlayerAgent * agent );
+
 };
 
 }
