@@ -141,9 +141,9 @@ public:
       \param width width(y-range) of new rectangle.
      */
     static
-    Rect2D make_with_center( const Vector2D & center,
-                             const double & length,
-                             const double & width )
+    Rect2D from_center( const Vector2D & center,
+                        const double & length,
+                        const double & width )
       {
           return Rect2D( center.x - length*0.5,
                          center.y - width*0.5,
@@ -152,13 +152,32 @@ public:
       }
 
     /*!
-      \brief create rectangle with 2 points. just call one of constructor.
+      \brief create rectangle with center point and size.
+      \param center_x x value of center point of new rectangle.
+      \param center_y y value of center point of new rectangle.
+      \param length length(x-range) of new rectangle.
+      \param width width(y-range) of new rectangle.
+     */
+    static
+    Rect2D from_center( const double & center_x,
+                        const double & center_y,
+                        const double & length,
+                        const double & width )
+      {
+          return Rect2D( center_x - length*0.5,
+                         center_y - width*0.5,
+                         length,
+                         width );
+      }
+
+    /*!
+      \brief create rectangle with 2 corner points. just call one of constructor.
       \param top_left top left vertex
       \param bottom_right bottom right vertex
     */
     static
-    Rect2D make_with_points( const Vector2D & top_left,
-                             const Vector2D & bottom_right )
+    Rect2D from_corners( const Vector2D & top_left,
+                         const Vector2D & bottom_right )
       {
           return Rect2D( top_left, bottom_right );
       }
