@@ -328,7 +328,8 @@ ParamMap::getParamPtrShortName( const std::string & short_name )
 
 */
 std::ostream &
-ParamMap::printHelp( std::ostream & os ) const
+ParamMap::printHelp( std::ostream & os,
+                     const bool with_default ) const
 {
     const std::string indent_nl = "\n        ";
 
@@ -354,8 +355,11 @@ ParamMap::printHelp( std::ostream & os ) const
             os << " <Value>";
         }
 
-        os << " : (DefaultValue=\"";
-        (*it)->printValue( os ) << "\")";
+        if ( with_default )
+        {
+            os << " : (DefaultValue=\"";
+            (*it)->printValue( os ) << "\")";
+        }
 
         if ( ! (*it)->description().empty() )
         {
