@@ -39,9 +39,13 @@
 #include <rcsc/types.h>
 
 #include <string>
+#include <vector>
 
 namespace rcsc {
 
+
+
+
 /*-------------------------------------------------------------------*/
 /*!
   \struct PlayerMessage;
@@ -53,16 +57,21 @@ struct PlayerMessage {
       \brief communication message type
     */
     enum Type {
-        BALL_INFO, // "b "
-        PASS_INFO, // "p "
-        OPPONENT_GOALIE_INFO, // "g "
-        OFFSIDE_LINE_INFO, // "o "
-        DEFENSE_LINE_INFO, // "d "
-        WAIT_REQUEST, // "w "
-        ATTACK_REQUEST, // "a "
-        INTERCEPT, // "i "
-        HEY_PASS, // "h "
-        ILLEGAL
+        UNKNOWN              = 0x00000000,
+        BALL_INFO            = 0x00000001, // "b "
+        PASS_INFO            = 0x00000002, // "p "
+        OPPONENT_GOALIE_INFO = 0x00000004, // "g "
+        OFFSIDE_LINE_INFO    = 0x00000008, // "o "
+        DEFENSE_LINE_INFO    = 0x00000010, // "d "
+        WAIT_REQUEST         = 0x00000020, // "w "
+        ATTACK_REQUEST       = 0x00000040, // "a "
+        INTERCEPT_INFO       = 0x00000080, // "i "
+        HEY_PASS             = 0x00000100, // "h "
+    };
+
+    struct Info {
+        std::string str_;
+        Type type_;
     };
 
     GameTime time_; //!< received time
@@ -70,20 +79,53 @@ struct PlayerMessage {
     int unum_; //!< sender's uniform number
     double dir_; //!< sender's direction
     std::string message_; //!< raw message string
-    Type type_;
+
+    std::vector< Info > info_;
 
     PlayerMessage()
         : time_( -1, 0 )
         , side_( NEUTRAL )
-        , unum_( 0 )
+        , unum_( Unum_Unknown )
         , dir_( 0.0 )
-        , type_( ILLEGAL )
       {
           message_.reserve( 10 );
       }
 
 };
 
+#if 0
+
+struct BallInfoMessage
+    : public PlayerMessage {
+
+    Vector2D ball_pos_;
+    Vector2D ball_vel_;
+
+    void update( WorldModel & world )
+      {
+          world.setHeardBallInfo( ball_pos_,
+                                  ball_vel_,
+                                  time_ );
+      }
+
+};
+
+struct PassInfoMessage
+    : public PlayerMessage {
+
+    int receiver_;
+    Vector2D pos_;
+
+    void update( WorldModel & world )
+      {
+          world.setHeardPassInfo( receiver_,
+                                  pos_,
+                                  time_ );
+      }
+};
+
+#endif
+
 /*-------------------------------------------------------------------*/
 /*!
   \struct FreeformMessage
@@ -418,36 +460,61 @@ public:
 private:
     /*
       \brief analyze v8 format message from teammate
-      \return true if successfully analyzed
     */
-    bool parseTeammateAudioV8();
+    void parseTeammateAudioV8();
+
+    /*!
+      \brief analyze ball info message from teammate
+      \param msg head of message body sent from teammate
+      \return size of read byte, 0 means the invalid message
+     */
+    int parseBallInfo( const char * msg );
 
     /*!
       \brief analyze pass message from teammate
       \param msg head of message body sent from teammate
-      \return true if successfully analyzed
+      \return size of read byte, 0 means the invalid message
     */
-    bool parsePassInfo( const char * msg );
+    int parsePassInfo( const char * msg );
 
     /*!
       \brief analyze opponent goalie message from teammate
       \param msg head of message body sent from teammate
-      \return true if successfully analyzed
+      \return size of read byte, 0 means the invalid message
     */
-    bool parseGoalieInfo( const char * msg );
+    int parseGoalieInfo( const char * msg );
 
+    /*!
+      \brief analyze offside line info message from teammate
+      \param msg head of message body sent from teammate
+      \return size of read byte, 0 means the invalid message
+    */
+    int parseOffsideLineInfo( const char * msg );
 
-    bool parseOffsideLineInfo( const char * msg );
+    /*!
+      \brief analyze defense line info message from teammate
+      \param msg head of message body sent from teammate
+      \return size of read byte, 0 means the invalid message
+    */
+    int parseDefenseLineInfo( const char * msg );
 
-    bool parseDefenseLineInfo( const char * msg );
+    //int parseWaitRequest( const char * msg );
 
-    //bool parseWaitRequest( const char * msg );
+    //int parseAttackRequest( const char * msg );
 
-    //bool parseAttackRequest( const char * msg );
+    /*!
+      \brief analyze interception info message from teammate
+      \param msg head of message body sent from teammate
+      \return size of read byte, 0 means the invalid message
+    */
+    int parseInterceptInfo( const char * msg );
 
-    bool parseInterceptInfo( const char * msg );
-
-    bool parseHeyPassInfo( const char * msg );
+    /*!
+      \brief analyze offside line info message from teammate
+      \param msg head of message body sent from teammate
+      \return size of read byte, 0 means the invalid message
+    */
+    int parseHeyPassInfo( const char * msg );
 
 };
 
