@@ -80,8 +80,8 @@ private:
     //! connection to Soccer Viewer
     boost::shared_ptr< UDPSocket > M_socket;
 
-    //! output file stream
-    std::ofstream M_file;
+    //! flag to check write mode
+    bool M_write_mode;
 
     //! main buffer to output all
     std::string M_main_buffer;
@@ -101,9 +101,6 @@ private:
     //! circle info to be drawn
     std::vector< Circle2D > M_circles;
 
-    //! freeform messag only written to disk.
-    std::string M_freeforms;
-
 
 public:
     /*!
@@ -125,14 +122,10 @@ public:
                   const int port );
 
     /*!
-      \brief open log file
-      \param teamname team name string used as log file name prefix
-      \param unum agent's uniform number used as log file name number
-      \param file_dir directory path that log file is put
+      \brief set write mode
     */
-    void open( const std::string & teamname,
-               const int unum,
-               const std::string & file_dir );
+    void setWriteMode( const bool on );
+
     /*!
       \brief output to stream or socket
       \param world const reference to the world mode object
@@ -245,22 +238,6 @@ public:
       \param circle circle object
      */
     void addCircle( const Circle2D & circle );
-
-    /*!
-      \brief set free form message
-      \param msg formated string
-    */
-    void addFreeform( char * msg, ... );
-
-    /*!
-      \brief set free form message
-      \param msg raw message string
-    */
-    void addFreeform( const char * msg )
-      {
-          M_freeforms += msg;
-          M_freeforms += '\n';
-      }
 };
 
 }
