@@ -127,6 +127,42 @@ FullstateSensor::parse( const char * msg,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+FullstateSensor::reverse()
+{
+    M_ball.pos_ *= -1.0;
+    M_ball.vel_ *= -1.0;
+
+    {
+        const PlayerCont::iterator end = M_left_team.end();
+        for ( PlayerCont::iterator it = M_left_team.begin();
+              it != end;
+              ++it )
+        {
+            it->pos_ *= -1.0;
+            it->vel_ *= -1.0;
+            it->body_ = AngleDeg::normalize_angle( it->body_ + 180.0 );
+        }
+    }
+
+    {
+        const PlayerCont::iterator end = M_right_team.end();
+        for ( PlayerCont::iterator it = M_right_team.begin();
+              it != end;
+              ++it )
+        {
+            it->pos_ *= -1.0;
+            it->vel_ *= -1.0;
+            it->body_ = AngleDeg::normalize_angle( it->body_ + 180.0 );
+        }
+    }
+
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 FullstateSensor::parseV8( const char * msg )
