@@ -37,7 +37,8 @@
 #include <rcsc/player/debug_client.h>
 #include <rcsc/player/player_config.h>
 #include <rcsc/player/see_state.h>
-#include <rcsc/client/basic_client.h>
+#include <rcsc/common/soccer_agent.h>
+#include <rcsc/timer.h>
 #include <rcsc/types.h>
 
 #include <boost/scoped_ptr.hpp>
@@ -62,7 +63,7 @@ struct PlayerAgentImpl;
   \brief basic player agent class
 */
 class PlayerAgent
-    : public BasicClient {
+    : public SoccerAgent {
 private:
 
     //! internal implementation object
@@ -317,14 +318,6 @@ public:
     bool doIntention();
 
 protected:
-    /*!
-      \brief get configuration set
-      \return reference to the configuration class object
-    */
-    PlayerConfig & getConfig()
-      {
-          return M_config;
-      }
 
     /*!
       \brief analyze command line options
@@ -335,32 +328,18 @@ protected:
       BasicClient::init(argc,argv) should be called in main().
     */
     virtual
-    bool doInit( CmdLineParser & cmd_parser );
+    bool initImpl( CmdLineParser & cmd_parser );
 
     /*!
-      \brief create connection to the server
+      \brief handle start event
+      \return status of start procedure.
 
-      This method is called on the top of BasicClient::run() method.
+      This method is called when client starts to run.
+      The concrete agent must connect to the server and send init command.
       Do NOT call this method by yourself!
-    */
+     */
     virtual
-    void doConnect();
-
-    /*!
-      \brief send init or reconnect command to server
-
-      init commad is sent in BasicClient's run() method
-      Do NOT call this method by yourself!
-    */
-    virtual
-    void sendInitCommand();
-
-    /*!
-      \brief send disconnection command message to server
-      and set the server status to end.
-    */
-    virtual
-    void sendByeCommand();
+    bool handleStart();
 
     /*!
       \brief handle server message event
@@ -372,13 +351,16 @@ protected:
     void handleMessage();
 
     /*!
-      \brief handle select timeout
+      \brief handle timeout event
+      \param timeout_count count of timeout without sensory message.
+      \param waited_msec elapsed milli seconds sinc last sensory message.
 
       This method is called from BasicClient::run() method.
       Do NOT call this method by yourself!
     */
     virtual
-    void handleTimeout();
+    void handleTimeout( const int timeout_count,
+                        const int waited_msec );
 
     /*!
       \brief handle exit event
@@ -394,6 +376,20 @@ public:
 
 private:
 
+    /*!
+      \brief send init or reconnect command to server
+
+      init commad is sent in BasicClient's run() method
+      Do NOT call this method by yourself!
+    */
+    void sendInitCommand();
+
+    /*!
+      \brief send disconnection command message to server
+      and set the server status to end.
+    */
+    void sendByeCommand();
+
     /*!
       \brief set debug output flags to logger
      */
@@ -581,16 +577,6 @@ private:
 
 protected:
 
-    /*!
-      \brief pure virtual method. init team info.
-
-      This method is used to set the team strategy etc.
-      This method is called at the end of analyzeInit() method.
-      So, do *not* call this method by yourself.
-    */
-    virtual
-    void initTeam() = 0;
-
     /*!
       \brief pure virtual method. register body action to ActionEffector.
 
@@ -599,7 +585,7 @@ protected:
       So, do *NOT* call this method by yourself.
     */
     virtual
-    void doAction() = 0;
+    void actionImpl() = 0;
 
     /*!
       \brief virtual method. register say action to ActionEffector
@@ -609,7 +595,7 @@ protected:
       So, do *not* call this method by yourself.
     */
     virtual
-    void doCommunication()
+    void communicationImpl()
       { }
 
     /*!
