@@ -56,6 +56,8 @@ private:
     //! validation count threshold value for M_body and M_face
     static int S_face_count_thr;
 
+    bool M_ghost; //!< ghost flag
+
     int  M_unum; //!< uniform number
     bool M_goalie; //!< goalie flag
 
@@ -68,6 +70,9 @@ private:
     int M_pos_count; //!< main accuracy counter
     int M_rpos_count; //!< relative coodinate accuracy counter
 
+    Vector2D M_prev_pos; //! previous seen global coodinate
+    int M_prev_pos_count; //! count from previous seen
+
     Vector2D M_vel; //!< global velocity
     int M_vel_count; //!< accuracy count
 
@@ -112,6 +117,16 @@ public:
                         const int face_thr );
 
     // ------------------------------------------
+
+    /*!
+      \brief check if this player is ghost object or not
+      \return true if this player may be ghost object
+     */
+    bool isGhost() const
+      {
+          return M_ghost;
+      }
+
     /*!
       \brief get player's uniform number
       \return uniform number. if unknown player, returned -1
@@ -170,6 +185,15 @@ public:
           return M_pos;
       }
 
+    /*!
+      \brief get global position accuracy
+      \return count from last observation
+    */
+    int posCount() const
+      {
+          return M_pos_count;
+      }
+
     /*!
       \brief get position relative to self position
       \return const reference to the point object
@@ -181,12 +205,22 @@ public:
       }
 
     /*!
-      \brief get global position accuracy
-      \return count from last observation
-    */
-    int posCount() const
+      \brief get the last seen position
+      \return const reference to the point object
+     */
+    const
+    Vector2D & prevPos() const
       {
-          return M_pos_count;
+          return M_prev_pos;
+      }
+
+    /*!
+      \brief get the number of cycles from previous seen
+      \return count from previous seen
+    */
+    int prevPosCount() const
+      {
+          return M_prev_pos_count;
       }
 
     /*!
@@ -341,6 +375,14 @@ public:
     */
     void update();
 
+    /*!
+      \brief set ghost flag
+     */
+    void setGhost()
+      {
+          M_ghost = true;
+      }
+
     /*!
       \brief update status using localized player info
       \param p localized player info
@@ -411,12 +453,14 @@ public:
     /*!
       \brief predicate functor to compare player's accuracy. reference version
     */
-    class ConfCmp
+    class CountCmp
         : public std::binary_function< PlayerObject, PlayerObject, bool > {
     public:
         result_type operator()( const first_argument_type & lhs,
                                 const second_argument_type & rhs ) const
           {
+              if ( lhs.goalie() ) return true;
+              if ( rhs.goalie() ) return false;
               return lhs.posCount() < rhs.posCount();
           }
     };
@@ -424,12 +468,14 @@ public:
     /*!
       \brief predicate functor to compare player's accuracy. pointer version
     */
-    class PtrConfCmp
+    class PtrCountCmp
         : public std::binary_function< PlayerObject *, PlayerObject *, bool > {
     public:
         result_type operator()( const first_argument_type lhs,
                                 const second_argument_type rhs ) const
           {
+              if ( lhs->goalie() ) return true;
+              if ( rhs->goalie() ) return false;
               return lhs->posCount() < rhs->posCount();
           }
     };
