@@ -41,11 +41,14 @@
 #include "ball_object.h"
 #include "logger.h"
 
-#include <rcsc/param/server_param.h>
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/server_param.h>
+#include <rcsc/common/player_type.h>
 #include <rcsc/soccer_math.h>
 #include <rcsc/math_util.h>
 
+#define DEBUG1
+//#define DEBUG
+
 namespace rcsc {
 
 const double SelfIntercept::MIN_TURN_THR = 12.5;
@@ -59,10 +62,11 @@ void
 SelfIntercept::predict( const int max_cycle,
                         std::vector< InterceptInfo > & self_cache ) const
 {
+#ifdef DEBUG
     dlog.addText( Logger::INTERCEPT,
                   "%s:%d: ------------- predict self ---------------"
                   ,__FILE__, __LINE__ );
-
+#endif
     predictOneStep( self_cache );
     predictLongStep( max_cycle, self_cache );
 }
@@ -152,8 +156,10 @@ SelfIntercept::predictNoDash( const bool goalie_mode,
 {
     const double next_ball_dist = next_ball_rel.r();
 
+    double ball_noise = M_world.ball().vel().r() * ServerParam::i().ballRand() * 1.4;
+
     // over control area
-    if ( next_ball_dist > control_area - 0.2 )
+    if ( next_ball_dist > control_area - 0.15 - ball_noise )
     {
         return false;
     }
@@ -285,7 +291,7 @@ SelfIntercept::predictOneDash( const double & control_area,
         = std::max( max_back_accel_x,
                     -0.5 * safety_power_back * M_world.self().dashRate() );
 
-#ifdef DEBUG
+#ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
                   "____1 dash: max_forward_accel = %f max_back_accel = %f",
                   max_forward_accel_x, max_back_accel_x );
@@ -303,9 +309,9 @@ SelfIntercept::predictOneDash( const double & control_area,
                                 true, // stamina safe
                                 self_cache ) )
     {
-#ifdef DEBUG
+#ifdef DEBUG1
         dlog.addText( Logger::INTERCEPT,
-                      "____1 dash. Success! and stamina safe" );
+                      "<--- 1 dash. Success! and stamina safe" );
 #endif
         return;
     }
@@ -318,14 +324,14 @@ SelfIntercept::predictOneDash( const double & control_area,
                                 false, // stamina not safe
                                 self_cache ) )
     {
-#ifdef DEBUG
+#ifdef DEBUG1
         dlog.addText( Logger::INTERCEPT,
-                      "____1 dash. Success! but not stamina safe" );
+                      "<--- 1 dash. Success! but not stamina safe" );
 #endif
         return;
     }
 
-#ifdef DEBUG
+#ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
                   "____1 dash. failure" );
 #endif
@@ -343,7 +349,7 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
                                       const bool stamina_safety,
                                       std::vector< InterceptInfo > & self_cache ) const
 {
-#ifdef DEBUG
+#ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
                   "______1 dash Adjust. back_acc_x = %.3f  next_ball_rel.x = %.3f"
                   "  forward_acc_x = %.3f",
@@ -361,7 +367,7 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
                                               next_ball_rel,
                                               back_accel_x,
                                               forward_accel_x );
-#ifdef DEBUG
+#ifdef DEBUG1
         dlog.addText( Logger::INTERCEPT,
                       "______1 dash Adjust(1). Best Trap. power = %.1f",
                       dash_power );
@@ -380,7 +386,7 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
         {
             // at least, reach the controllale distance
             dash_power = forward_accel_x / M_world.self().dashRate();
-#ifdef DEBUG
+#ifdef DEBUG1
             dlog.addText( Logger::INTERCEPT,
                           "_____1 dash Adjust(2). Not Best. next_ball_dist = %.3f  power = %.1f",
                           enable_ball_dist, dash_power );
@@ -398,7 +404,7 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
         if ( enable_ball_dist < control_area - 0.2 )
         {
             dash_power = back_accel_x / M_world.self().dashRate();
-#ifdef DEBUG
+#ifdef DEBUG1
             dlog.addText( Logger::INTERCEPT,
                           "______1 dash Adjust(3).Not Best next_ball_dist = %.3f  power = %.1f",
                           enable_ball_dist, dash_power );
@@ -412,7 +418,7 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
          && next_ball_rel.x < forward_accel_x )
     {
         dash_power = next_ball_rel.x / M_world.self().dashRate();
-#ifdef DEBUG
+#ifdef DEBUG1
         dlog.addText( Logger::INTERCEPT,
                       "______1 dash Adjust(4).Not Best. just adjust X. dash power = %.1f",
                       dash_power );
@@ -434,7 +440,7 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
             }
         }
         self_cache.push_back( InterceptInfo( mode, 0, 1, dash_power ) );
-#ifdef DEBUG
+#ifdef DEBUG1
         dlog.addText( Logger::INTERCEPT,
                       "--->Success! 1 dash Adjust. register new dash power = %f  mode = %d",
                       dash_power, mode );
@@ -442,7 +448,7 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
         return true;
     }
 
-#ifdef DEBUG
+#ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
                   "______1 dash Adjust. Failed" );
 #endif
@@ -459,12 +465,12 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
                                         const double & max_back_accel_x,
                                         const double & max_forward_accel_x ) const
 {
-    double best_ctrl_dist
+    const double best_ctrl_dist
         = M_world.self().playerType().playerSize()
         + 0.5 * ( M_world.self().playerType().kickableMargin()
                   + ServerParam::i().ballSize() );
 
-#ifdef DEBUG
+#ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
                   "________1 dash Best Trap. BestCtrlDist = %.3f, next_ball_y = %.3f",
                   best_ctrl_dist, next_ball_rel.y );
@@ -474,7 +480,7 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
     // just put the ball on player's side
     if ( next_ball_rel.absY() > best_ctrl_dist )
     {
-#ifdef DEBUG
+#ifdef DEBUG1
         dlog.addText( Logger::INTERCEPT,
                       "________1 dash Best Trap.  just put the ball on side" );
 #endif
@@ -490,16 +496,27 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
     // case put ball back
     required_accel_x[1]
         = next_ball_rel.x
-        + std::sqrt( std::pow( best_ctrl_dist, 2.0 )
+        + std::sqrt( std::pow( best_ctrl_dist - 0.1, 2.0 )
                      - std::pow( next_ball_rel.y, 2.0 ) );
+#if 0
     // nearest side has priority
     if ( std::fabs( required_accel_x[1] )
          < std::fabs( required_accel_x[0] ) )
     {
         std::swap( required_accel_x[0], required_accel_x[1] );
     }
+#else
+    if ( M_world.self().body().abs() < 45.0 )
+    {
+        // forward dash has priority if player's body face to opponent side
+        if ( required_accel_x[1] > required_accel_x[0] )
+        {
+            std::swap( required_accel_x[0], required_accel_x[1] );
+        }
+    }
+#endif
 
-#ifdef DEBUG
+#ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
                   "________1 dash: best_ctrl accel_x[0] = %f  accel_x[1] = %f",
                   required_accel_x[0], required_accel_x[1] );
@@ -510,7 +527,7 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
         if ( required_accel_x[i] >= 0.0
              && max_forward_accel_x > required_accel_x[i] )
         {
-#ifdef DEBUG
+#ifdef DEBUG1
             dlog.addText( Logger::INTERCEPT,
                           "________1 dash Best Trap.  best trap."
                           " forward dash[%d]. x = %f",
@@ -521,7 +538,7 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
         if ( required_accel_x[i] < 0.0
              && max_back_accel_x < required_accel_x[i] )
         {
-#ifdef DEBUG
+#ifdef DEBUG1
             dlog.addText( Logger::INTERCEPT,
                           "________1 dash Best Trap.  best trap."
                           " back dash[%d]. x = %f",
@@ -530,7 +547,7 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
             return required_accel_x[i] / M_world.self().dashRate();
         }
     }
-#ifdef DEBUG
+#ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
                   "________1 dash Best Trap. Failed" );
 #endif
@@ -598,8 +615,8 @@ SelfIntercept::predictLongStep( const int max_cycle,
 #endif
             break;;
         }
-        if ( ball_pos.absX() > ServerParam::i().pitchHalfLength() + 5.0
-             || ball_pos.absY() > ServerParam::i().pitchHalfWidth() + 5.0
+        if ( ball_pos.absX() > ServerParam::i().pitchHalfLength() + 10.0
+             || ball_pos.absY() > ServerParam::i().pitchHalfWidth() + 10.0
              )
         {
 #ifdef DEBUG
@@ -669,10 +686,9 @@ SelfIntercept::predictLongStep( const int max_cycle,
     // not registered any interception
     if ( ! found )
     {
-#ifdef DEBUG
         dlog.addText( Logger::INTERCEPT,
-                      "__ Failed to predict? register ball final point" );
-#endif
+                      "%s: SelfIntercept. failed to predict? register ball final point",
+                      __FILE__ );
         predictFinal( self_cache );
     }
 }
@@ -1141,17 +1157,17 @@ SelfIntercept::canReachAfterDash( const int n_turn,
 
     //////////////////////////////////////////////////////////
     // when cycle is small, do strict check
-    if ( n_dash <= 4 )
+    if ( n_turn + n_dash <= 4 )
     {
         // tmp_pos is relative to playerPos() --> tmp_pos.r() == player_travel
         double player_travel = tmp_pos.r();
         double player_noise = player_travel * ServerParam::i().playerRand() * 0.5;
         double last_ball_dist = ball_rel.dist( tmp_pos );
         double buf = 0.0;
-        if ( n_turn > 0 ) buf += 0.1;
+        if ( n_turn > 0 ) buf += 0.2;
         buf += player_noise;
         buf += ball_noise;
-        if ( last_ball_dist < control_area - buf )//0.2 )
+        if ( last_ball_dist < control_area - buf )
         {
 #ifdef DEBUG
             dlog.addText( Logger::INTERCEPT,
