@@ -466,14 +466,14 @@ bool
 TrainerAgent::analyzeCycle( const char * msg,
                             const bool by_see_global )
 {
-    char buf[16];
+    char buf[128];
     long cycle = 0;
 
     if ( std::sscanf( msg, "(%s %ld ",
                       buf, &cycle ) != 2
-         && std::sscanf( msg, "(hear (%[^()]) %ld ",
+         && std::sscanf( msg, "(hear (%127[^()]) %ld ",
                          buf, &cycle ) != 2
-         && std::sscanf( msg, "(hear %s %ld ",
+         && std::sscanf( msg, "(hear %127s %ld ",
                          buf, &cycle ) != 2 )
     {
         std::cerr << "trainer: time parse error msg=["
@@ -518,19 +518,22 @@ TrainerAgent::analyzeSeeGlobal( const char * msg )
 void
 TrainerAgent::analyzeHear( const char * msg )
 {
+    std::cerr << "Trainer hear " << msg << std::endl;
     if ( ! analyzeCycle( msg, false ) )
     {
         return;
     }
 
     long cycle;
-    char sender[32];
+    char sender[128];
 
-    if ( std::sscanf( msg, "(hear %ld %s ",
+    if ( std::sscanf( msg, "(hear %ld (%127[^()]) ",
                       &cycle, sender ) != 2
-         && std::sscanf( msg, "(hear (%[^()]) %ld ",
+         && std::sscanf( msg, "(hear %ld %s ",
+                         &cycle, sender ) != 2
+         && std::sscanf( msg, "(hear (%127[^()]) %ld ",
                          sender, &cycle ) != 2
-         && std::sscanf( msg, "(hear %s %ld ",
+         && std::sscanf( msg, "(hear %127s %ld ",
                          sender, &cycle ) != 2 )
     {
         std::cerr << "trainer: "
@@ -547,7 +550,6 @@ TrainerAgent::analyzeHear( const char * msg )
     }
     else
     {
-        //std::cerr << "hear sender = [" << sender << ']' << std::endl;
         // player message
     }
 }
@@ -562,8 +564,8 @@ TrainerAgent::analyzeHearReferee( const char * msg )
     long cycle;
     char mode[512];
 
-    if ( std::sscanf( msg, "(hear %ld referee %[^)]", &cycle, mode ) != 2
-         && std::sscanf( msg, "(hear referee %ld %[^)]", &cycle, mode ) != 2 )
+    if ( std::sscanf( msg, "(hear %ld referee %511[^)]", &cycle, mode ) != 2
+         && std::sscanf( msg, "(hear referee %ld %511[^)]", &cycle, mode ) != 2 )
     {
         std::cerr << "trainer: "
                   << M_impl->current_time_
@@ -620,7 +622,7 @@ TrainerAgent::analyzeChangePlayerType( const char * msg )
 
     std::memset( teamname, 0, 32 );
 
-    if ( std::sscanf( msg, "(ok change_player_type %[^ ] %d %d)",
+    if ( std::sscanf( msg, "(ok change_player_type %31[^ ] %d %d)",
                       teamname, &unum, &type ) == 3 )
     {
         if ( world().teamNameLeft() == teamname )
@@ -767,7 +769,7 @@ TrainerAgent::analyzeTeamNames( const char * msg )
     char left[32], right[32];
 
     int n = std::sscanf( msg,
-                         "(ok team_names (team l %[^)]) (team r %[^)]))",
+                         "(ok team_names (team l %31[^)]) (team r %31[^)]))",
                          left, right );
     if ( n == 2 )
     {
