@@ -30,16 +30,13 @@
 /////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <cstdio>
-#include <cstring>
-#include <cstdlib>
-#include <iostream>
-
 #include <rcsc/game_mode.h>
 
+#include <iostream>
+#include <cstring>
 
 namespace {
 
@@ -344,7 +341,7 @@ GameMode::isOurSetPlay( const SideID ourside ) const
 
 */
 PlayMode
-GameMode::getPlayMode() const
+GameMode::getServerPlayMode() const
 {
     switch ( type() ) {
     case BeforeKickOff:
