@@ -33,7 +33,6 @@
 #include "config.h"
 #endif
 
-#include <cstdlib>
 #include <cstring>
 #include <iostream>
 #include <fstream>
@@ -57,7 +56,7 @@ public:
     TeamNameLoader( std::istream & is );
 
     const
-    std::vector< std::string > & getTeams() const
+    std::vector< std::string > & teams() const
       {
           return M_teams;
       }
@@ -138,7 +137,7 @@ public:
 
     void create( int total_teams );
 
-    const MatchCont & getMatchList() const
+    const MatchCont & matchList() const
       {
           return M_match_list;
       }
@@ -200,10 +199,7 @@ Scheduler::create( const int total_teams )
 
         int left = i;
         int right = k[down] + i;
-        if ( rand() % 2 )
-        {
-            std::swap( left, right );
-        }
+
         M_match_list.push_back( std::make_pair( left, right ) );
     }
 }
@@ -231,7 +227,7 @@ Scheduler::print( std::ostream & os ) const
 /*---------------------------------------------------------------*/
 /*
   Usage:
-  $ rclmscheduler <TeamListedFile>
+  $ rclmscheduler < TeamListedFile
 */
 void
 usage()
@@ -246,39 +242,46 @@ usage()
 int
 main( int argc, char** argv )
 {
-    std::srand( std::time( NULL ) );
-
+    for ( int i = 1; i < argc; ++i )
+    {
+        if ( ! std::strcmp( argv[i], "--help" )
+             || ! std::strcmp( argv[i], "-h" ) )
+        {
+            usage();
+            return 1;
+        }
+    }
 
     TeamNameLoader loader( std::cin );
     //loader.print( std::cout );
     //std::cout << " -------------------------------- " << std::endl;
 
-    if( loader.getTeams().empty() )
+    if( loader.teams().empty() )
     {
-        usage();
         std::cerr << "Team size is empty!" << std::endl;
+        usage();
         return 1;
     }
 
-    Scheduler scheduler( loader.getTeams().size() );
+    Scheduler scheduler( loader.teams().size() );
 
     //scheduler.print( std::cout );
     //std::cout << " -------------------------------- " << std::endl;
 
-    std::cerr << "The number of teams : " << loader.getTeams().size() << std::endl;
-    std::cerr << "The number of matches : " << scheduler.getMatchList().size() << std::endl;
+    std::cerr << "The number of teams : " << loader.teams().size() << std::endl;
+    std::cerr << "The number of matches : " << scheduler.matchList().size() << std::endl;
 
     //int count = 0;
-    const Scheduler::MatchCont::const_iterator end = scheduler.getMatchList().end();
-    for ( Scheduler::MatchCont::const_iterator it =  scheduler.getMatchList().begin();
+    const Scheduler::MatchCont::const_iterator end = scheduler.matchList().end();
+    for ( Scheduler::MatchCont::const_iterator it =  scheduler.matchList().begin();
           it != end;
           ++it )
     {
         std::cout
             // << ++count << ": "
-            << loader.getTeams().at( it->first )
+            << loader.teams().at( it->first )
             << " vs "
-            << loader.getTeams().at( it->second )
+            << loader.teams().at( it->second )
             << std::endl;
     }
 
